@@ -3,7 +3,7 @@
 
 import os
 import requests
-from requests_kerberos import HTTPKerberosAuth, OPTIONAL
+from requests_gssapi import HTTPKerberosAuth, OPTIONAL # noqa
 
 
 if __name__ == '__main__':
@@ -18,4 +18,3 @@ if __name__ == '__main__':
             r.headers.get("WWW-Authenticate").startswith("Negotiate")):
         raise ValueError('Spnego failed - WWW-Authenticate Negotiate header '
                          'missing')
-
